# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package Scmbug::Connection;

use strict;
use IO::Socket;
use Data::Dumper;
use Scmbug::Activity;
use Scmbug::Error;
use Scmbug::Common;

my $connection_configuration;



# Constructor
sub new {
    my $type = shift;
    my $self = {};
    return bless $self, $type;
}



sub port {
    my $self = shift;
    my $port = shift;

    $connection_configuration->{ port } = $port;
}



sub location {
    my $self = shift;
    my $location = shift;

    $connection_configuration->{ location } = $location;
}



sub process_activity {
    my $self = shift;
    my ( $activity, $policies ) = ( @_ );
    my $request = "";

    # Prepare the request to the daemon
    notify( "Processing activity\n" );    

    # Username submitting the request
    $request .= $activity->user() . "\n";

    # Product name
    $request .= $activity->product_name() . "\n";

    # Repository
    $request .= $activity->repository() . "\n";

    # Requested activity
    $request .= $activity->name() . "\n";

    if ( $activity->name() eq $ACTIVITY_COMMIT ) {
	$request .= $activity->branch_name() . "\n";
	$request .= process_policies_common( $policies );
	$request .= process_activity_commit( $activity );
    } elsif ( $activity->{ name } eq $ACTIVITY_VERIFY ) {
	$request .= $activity->branch_name() . "\n";
	$request .= process_policies_common( $policies );
	$request .= process_activity_verify( $activity );
    } elsif ( $activity->{ name } eq $ACTIVITY_TAG ) {
	$request .= $activity->branch_name() . "\n";
	$request .= process_policies_common( $policies );
	$request .= process_activity_tag( $activity );
    } elsif ( $activity->{ name } eq $ACTIVITY_GET_VDD ) {
	$request .= process_activity_get_vdd( $activity );
    } elsif ( $activity->{ name } eq $ACTIVITY_GET_BUGS ) {
	$request .= process_activity_get_bugs( $activity );
    } else  {
	scmbug_error( $GLUE_ERROR_INVALID_ACTIVITY, "'" . $activity->name() . "' is an invalid integration activity.\n" );
    }

    # Log message
    $request .= $activity->log_message() . "\n";

    # End of request token
    $request .= $PRODUCT_END_TOKEN . "\n";

    return $self->submit_request( $request, 0 );
}



sub process_activity_commit
{
    my $activity = shift;

    return process_activity_common_verify_commit( $activity );
}



sub process_activity_verify
{
    my $activity = shift;

    return process_activity_common_verify_commit( $activity );
}



sub process_activity_tag
{
    my $activity = shift;
    my $request = "";

    # Send the tagging operation type
    $request .= $activity->{ type }->{ tag }->{ operation };
    $request .= "\n";

    # Send the tag name
    $request .= $activity->{ type }->{ tag }->{ name };
    $request .= "\n";

    # Comma separated list of ids
    $request .= join( ",", @{ $activity->ids() } ) . "\n";

    # Send the list of filename, file revision pairs
    foreach my $file ( sort keys %{ $activity->{ files } } ) {
	# Filename
	$request .= $file;
	$request .= "\n";
	# File revision
	$request .= $activity->{ files }->{ $file };
	$request .= "\n";
    }

    # Send the end of affected files token
    $request .= $PRODUCT_AFFECTED_FILES_END_TOKEN . "\n";;

    return $request;
}



sub process_activity_get_vdd
{
    my $activity = shift;
    my $request = "";

    $request .= $activity->{ from_version } . "\n"; 
    $request .= $activity->{ to_version } . "\n"; 
    $request .= $activity->{ from_label_date } . "\n"; 
    $request .= $activity->{ to_label_date } . "\n"; 

    # Send the end of vdd query token
    $request .= $PRODUCT_VDD_QUERY_END_TOKEN . "\n";;

    return $request;
}

sub process_activity_get_bugs
{
    my $activity = shift;
    my $request = "";

    $request .= $activity->{ bugs } . "\n";

    # Send the end of get bugs query token
    $request .= $PRODUCT_GET_BUGS_QUERY_END_TOKEN . "\n";;

    return $request;

}


sub process_activity_common_verify_commit
{
    my $activity = shift;
    my $request = "";

    # Comma separated list of ids
    $request .= join( ",", @{ $activity->ids() } ) . "\n";

    # For each file, report the old version, new version, and filename
    foreach my $file ( sort keys %{ $activity->{ files } } ) {
	$request .= $activity->{ files }->{ $file }->{ old_version };
	$request .= "\n";
	$request .= $activity->{ files }->{ $file }->{ new_version };
	$request .= "\n";
	$request .= $file;
	$request .= "\n";
    }
    
    # Send the end of affected files token
    $request .= $PRODUCT_AFFECTED_FILES_END_TOKEN . "\n";;

    return $request;
}



sub process_policies_common
{
    my ($policies) = ( @_ );
    my $request = "";

    # Policy presence_of_bug_ids
    $request .= $policies->{ presence_of_bug_ids }->{ value } . "\n";

    # Policy valid_bug_owner
    $request .= $policies->{ valid_bug_owner }->{ enabled } . "\n";

    # Policy open_bug_state
    $request .= $policies->{ open_bug_state }->{ enabled } . "\n";

    # Policy product_name
    $request .= $policies->{ valid_product_name }->{ enabled } . "\n";

    # Policy mail_on_success
    $request .= $policies->{ mail_on_success }->{ enabled } . "\n";
    $request .= $policies->{ mail_on_success }->{ mail_on_label } . "\n";
    $request .= $policies->{ mail_on_success }->{ values }->{ To } . "\n";
    $request .= $policies->{ mail_on_success }->{ values }->{ From } . "\n";
    $request .= $policies->{ mail_on_success }->{ values }->{ Smtp } . "\n";
    $request .= $policies->{ mail_on_success }->{ mail_also_appears_from_scm_user } . "\n";
    $request .= $policies->{ mail_on_success }->{ mail_bug_owner } . "\n";
    $request .= $policies->{ mail_on_success }->{ mail_bug_reporter } . "\n";
    $request .= $policies->{ mail_on_success }->{ mail_bug_monitors } . "\n";
    $request .= $policies->{ mail_on_success }->{ mail_scm_user } . "\n";
    $request .= $policies->{ mail_on_success }->{ mail_product_owners } . "\n";

    return $request;
}



sub submit_request {
    my ( $self, $request, $only_check_if_daemon_is_alive_and_well ) = ( @_ );
    my ( $location, $port );

    $location = $connection_configuration->{ location };
    $port = $connection_configuration->{ port };

    if ( !defined ( $location) ) {
	scmbug_error( $GLUE_ERROR_NO_CONNECTION_LOCATION, "No connection location supplied\n" );
    }
    if ( !defined ( $port) ) {
	scmbug_error( $GLUE_ERROR_NO_CONNECTION_PORT, "No connection port number supplied\n" );
    }

    my $socket = IO::Socket::INET->new( Proto => "tcp",
					PeerAddr => $location,
					PeerPort => $port,
					Timeout => 3 );
    if ( !defined ( $socket ) ) {
	scmbug_error( $GLUE_ERROR_UNABLE_TO_CONNECT_TO_DAEMON, "Unable to connect to integration daemon at '$location', port '$port'. Is the daemon running ?\n");
    }
    $socket->autoflush(1);

    # Read the banner from the daemon
    my $line;
    $line = <$socket>;

    # Verify that a matching glue and daemon are used
    my ( $daemon_name, $daemon_version, $protocol_number ) = split( " ", $line );
    if ( $protocol_number ne $PRODUCT_PROTOCOL_NUMBER ) {
	scmbug_error( $GLUE_ERROR_INVALID_PROTOCOL_NUMBER, "Connection version $PRODUCT_VERSION uses communication protocol $PRODUCT_PROTOCOL_NUMBER, which is incompatible with daemon version $daemon_version using communication protocol $protocol_number. Please upgrade accordingly.\n" );
    }

    if ( $only_check_if_daemon_is_alive_and_well == 1) {
	close ( $socket );
	return;
    }

    # Submit the request
    notify( "Submitting request\n" );
    print $socket $request . "\n";

    # Retrieve the request response
    my $response = "";
    $response = <$socket>;

    # If no response was received, format the notice correctly.
    if ( !defined( $response ) ) {
	notify( "Request submission was not replied to\n" );
    } else {
	notify( "Request submission was $response" );
    }


    # Retrieve any additional information returned
    my $ret_message = "";
    while ( $line = <$socket> ) {
	$ret_message .= $line;
    }

    if ( !defined( $response ) ) {
	# No response was received. The daemon-spawned connection
	# handler exited abnormally
	scmbug_error( $GLUE_ERROR_DAEMON_DIED, "The daemon-spawned connection handler exited abnormally. Please check the daemon error logs.\n" );
    } elsif ( $response !~ m/$DAEMON_REQUEST_RESULT_ACCEPTED/ ) {
	# The request was denied, need to exit with a non-zero
	# value. Inform why the request was denied
	scmbug_error( $GLUE_ERROR_ACTIVITY_FAILED, $ret_message );
    } else {
	# The request was accepted
	close ($socket);
	
	return $ret_message;
    }
}



1;
